package org.cebs.corep.xbrl.action;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.UnavailableException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import de.bundesbank.util.webgui.struts.BasisActionServlet;
import de.bundesbank.util.webgui.struts.IUseCaseObject;
import de.bundesbank.util.webgui.struts.Statics;

/**
 * Das <code>EuroActionServlet</code> ist der zentrale Einstiegspunkt in die
 * <strong>Euro-Evidenz</strong> Applikation. Es nimmt HTTP-Requests vom Browser des
 * Benutzers entgegen und wandelt sie in korrespondierende Aufrufe von
 * <code>EuroAction</code>-Klassen um. Die Action-Klassen beinhalten die Ablauf-
 * steuerung der Anwendung. Sie geben dem Servlet nach ihrer Abarbeitung jeweils
 * eine <code>EuroActionForward</code>-Instanz zurueck, die die Referenz auf eine
 * JSP-Seite beinhaltet, die zum Schluss aufgerufen werden soll.
 * <br />
 * Das <code>EuroActionServlet</code> erbt vom <code>ActionServlet</code> aus dem
 * Struts-Framework und stellt darueberhinaus erweiterte Funktionalitaeten fuer
 * Euro-Evidenz zur Verfuegung.
 * <br />
 * <b>ToDo:</b><ul>
 *  <li />...
 * </ul>
 *
 * @author    Thomas Brll
 */
public class CorepActionServlet extends BasisActionServlet
{

  /** Parameter Clipboard */
  public static final String ACTION_PARAMETER_CLIPBOARD = "bakisClipboard";

  /*statisch, da unter umstaenden mehrere Instanzen dieser Klasse angelegt und damzufolge initialisiert werden.
   * ist pro VM jedoch nur einmal notwendig.
   */
  private static InitAction initAction = null;

  private static CorepActionServlet instance = null;

  /** Default Konstruktor. */
  public CorepActionServlet()
  {
    setInstance(this);
  }

  /** Destroy-Methode im Lebenszyklus des Servlets. Ruft die Destroy-Methode des ActionServlets auf und loescht alle Listboxen aus dem ServletContext. */
  public void destroy()
  {
    initAction = null;
    super.destroy();
  }

  /**
   * Initialisiert das EuroActionServlet beim Starten des Servlet-Containers
   * <br />
   *
   * @exception  ServletException                
   */
  public void init() throws ServletException
  {

    // Init-Methode des Struts ActionServlets aufrufen
    super.init();
    if (initAction == null)
    {
      initAction = new InitAction();
      initAction.setServlet(this);

      try
      {
        initAction.prepareForServletUse();
        initAction.performInit();
      }
      catch (Exception e)
      {
        initAction = null;
        String msg = "Fehler bei der Initialisierung der Anwendung !";
        log(msg, e);
        throw new UnavailableException(msg + e.toString());
      }
    }
    else
    {
      getServletContext().setAttribute(org.apache.struts.action.Action.SERVLET_KEY, "*.start");
    }

  }

  /**
   * Diese Methode ueberschreibt die Originalversion aus dem
   * Struts <code>ActionServlet</code>. Ntig fr das Clipboard.
   *
   * @param  action             Action
   * @param  mapping            ActionMapping
   * @param  formInstance       ActionForm
   * @param  request            HttpServletRequest
   * @param  response           HttpServletResponse
   * @return                    ActionForward
   * @throws  IOException
   * @throws  ServletException
   */
  protected ActionForward processActionPerform(Action action, ActionMapping mapping, ActionForm formInstance, HttpServletRequest request, HttpServletResponse response)
    throws IOException, ServletException
  {
    int fehlerTyp = 0;
    try
    {
      if (formInstance != null)
      {
        IUseCaseObject useCase = (IUseCaseObject)formInstance;
        fehlerTyp = useCase.getFehlerTyp();
      }
    }
    catch (ClassCastException cce)
    {
      // ignorieren, wenn es sich bei der erzuegten Instanz nicht um ein
      // UseCase Form Objekt handelt
    }

    if (fehlerTyp > 0)
    {
      if (fehlerTyp == Statics.FEHLER_FORM_PARAMETER_NO_USECASE)
      {
        // Wenn kein UseCase Objekt gefunden wird, auf entsprechende Fehlerseite
        return mapping.findForward(Statics.FORWARD_NO_USECASE_ERROR);
      }
      else
      { // Wenn falsches Tooken, auf entsprechende Fehlerseite	
        return mapping.findForward(Statics.FORWARD_INVALID_TOKEN_ERROR);
      }
    }
    else
    {

      processNoCache(mapping, response);

      ActionForward forward = super.processActionPerform(action, mapping, formInstance, request, response);

      String parameter = getParameterValue(mapping, ACTION_PARAMETER_CLIPBOARD);

      if (parameter != null && _clipboard != null)
      {
        _clipboard.storeClipboard(request, formInstance, parameter, getParameterValue(mapping, Statics.ACTION_PARAMETER_CLIPBOARD_DEFAULT));
      }

      saveFormBean(mapping, formInstance, request, response);

      return (forward);
    }
  }

  /**
   * Diese Methode ueberschreibt die Originalversion aus dem
   * Struts <code>ActionServlet</code>. Hier koennen Vorab-
   * verarbeitungen vorgenommen werden, die vor dem Aufruf der
   * eigentlichen Action durchgefuehrt werden.
   *
   * @param  request            HttpServletRequest
   * @param  response           HttpServletResponse
   * @return                    boolean
   * @throws  IOException
   * @throws  ServletException
   */
  protected boolean processPreprocess(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException
  {

    return super.processPreprocess(request, response);
  }

  /**
   * Gets the instance
   *
   * @return    Returns a EuroActionServlet
   */
  public static CorepActionServlet getInstance()
  {
    return instance;
  }

  /**
   * Sets the instance
   *
   * @param  aInstance  Der neue instance Wert
   */
  public static void setInstance(CorepActionServlet aInstance)
  {
    instance = aInstance;
  }

}
